@@ -18,8 +18,8 @@ from ..compat import ordered_dict_class
 from ..exceptions import XMLSchemaAttributeError, XMLSchemaTypeError, XMLSchemaValueError
 from ..qnames import XSD_ANY_SIMPLE_TYPE, XSD_SIMPLE_TYPE, XSD_ATTRIBUTE_GROUP, \
     XSD_COMPLEX_TYPE, XSD_RESTRICTION, XSD_EXTENSION, XSD_SEQUENCE, XSD_ALL, \
-    XSD_CHOICE, XSD_ATTRIBUTE, XSD_ANY_ATTRIBUTE, XSD_ASSERT, get_namespace, \
-    get_qname, is_not_xsd_annotation
+    XSD_CHOICE, XSD_ATTRIBUTE, XSD_ANY_ATTRIBUTE, XSD_ASSERT, XSD_NOTATION_TYPE, \
+    get_namespace, get_qname, is_not_xsd_annotation
 from ..helpers import get_xsd_form_attribute
 from ..namespaces import XSI_NAMESPACE
 
@@ -54,6 +54,7 @@ class XsdAttribute(XsdComponent, ValidationMixin):
     qualified = False
     default = None
     fixed = None
+    inheritable = False  # For XSD 1.1 attributes, always False for XSD 1.0 attributes.
 
     def __init__(self, elem, schema, parent):
         super(XsdAttribute, self).__init__(elem, schema, parent)
@@ -90,7 +91,7 @@ class XsdAttribute(XsdComponent, ValidationMixin):
                 xsd_attribute = self.maps.lookup_attribute(self.name)
             except LookupError:
                 self.parse_error("unknown attribute %r" % self.name)
-                self.type = self.maps.lookup_type(XSD_ANY_SIMPLE_TYPE)
+                self.type = self.any_simple_type
             else:
                 self.ref = xsd_attribute
                 self.type = xsd_attribute.type
@@ -149,13 +150,13 @@ class XsdAttribute(XsdComponent, ValidationMixin):
                 type_qname = self.schema.resolve_qname(attrib['type'])
             except (KeyError, ValueError, RuntimeError) as err:
                 self.parse_error(err)
-                xsd_type = self.maps.lookup_type(XSD_ANY_SIMPLE_TYPE)
+                xsd_type = self.any_simple_type
             else:
                 try:
                     xsd_type = self.maps.lookup_type(type_qname)
                 except LookupError as err:
                     self.parse_error(err)
-                    xsd_type = self.maps.lookup_type(XSD_ANY_SIMPLE_TYPE)
+                    xsd_type = self.any_simple_type
 
                 if child is not None and child.tag == XSD_SIMPLE_TYPE:
                     self.parse_error("ambiguous type definition for XSD attribute")
@@ -167,7 +168,7 @@ class XsdAttribute(XsdComponent, ValidationMixin):
             xsd_type = self.schema.BUILDERS.simple_type_factory(child, self.schema, self)
         else:
             # Empty declaration means xsdAnySimpleType
-            xsd_type = self.maps.lookup_type(XSD_ANY_SIMPLE_TYPE)
+            xsd_type = self.any_simple_type
 
         try:
             self.type = xsd_type
@@ -238,6 +239,15 @@ class XsdAttribute(XsdComponent, ValidationMixin):
         if not text and self.default is not None:
             text = self.default
 
+        if self.type.is_notation():
+            if self.type.name == XSD_NOTATION_TYPE:
+                msg = "cannot validate against xs:NOTATION directly, " \
+                      "only against a subtype with an enumeration facet"
+                yield self.validation_error(validation, msg, text, **kwargs)
+            elif not self.type.enumeration:
+                msg = "missing enumeration facet in xs:NOTATION subtype"
+                yield self.validation_error(validation, msg, text, **kwargs)
+
         if self.fixed is not None:
             if text is None:
                 text = self.fixed
@@ -293,7 +303,6 @@ class Xsd11Attribute(XsdAttribute):
           Content: (annotation?, simpleType?)
         </attribute>
     """
-    inheritable = False
     _target_namespace = None
 
     @property
@@ -472,12 +481,12 @@ class XsdAttributeGroup(MutableMapping, XsdComponent, ValidationMixin):
                     attribute_group_refs.append(attribute_group_qname)
 
                     try:
-                        base_attributes = self.maps.lookup_attribute_group(attribute_group_qname)
+                        base_attrs = self.maps.lookup_attribute_group(attribute_group_qname)
                     except LookupError:
                         self.parse_error("unknown attribute group %r" % child.attrib['ref'], elem)
                     else:
-                        if not isinstance(base_attributes, tuple):
-                            for name, attr in base_attributes.items():
+                        if not isinstance(base_attrs, tuple):
+                            for name, attr in base_attrs.items():
                                 if name not in attributes:
                                     attributes[name] = attr
                                 elif name is not None:
@@ -531,6 +540,9 @@ class XsdAttributeGroup(MutableMapping, XsdComponent, ValidationMixin):
                         base_attr.type.normalize(base_attr.fixed):
                     self.parse_error("Attribute %r: derived attribute "
                                      "has a different fixed value" % name)
+                if base_attr.inheritable is not attr.inheritable:
+                    msg = "Attribute %r: attribute 'inheritable' value change in restriction"
+                    self.parse_error(msg % name)
 
             if self.redefine is not None:
                 pass  # In case of redefinition do not copy base attributes
@@ -600,7 +612,7 @@ class XsdAttributeGroup(MutableMapping, XsdComponent, ValidationMixin):
             if k is not None and v.use == 'required':
                 yield k
 
-    def iter_predefined(self, use_defaults=True):
+    def iter_value_constraints(self, use_defaults=True):
         if use_defaults:
             for k, v in self._attribute_group.items():
                 if k is None:
@@ -633,11 +645,16 @@ class XsdAttributeGroup(MutableMapping, XsdComponent, ValidationMixin):
         kwargs['level'] = kwargs.get('level', 0) + 1
         use_defaults = kwargs.get('use_defaults', True)
 
-        additional_attrs = [(k, v) for k, v in self.iter_predefined(use_defaults) if k not in attrs]
+        additional_attrs = [
+            (k, v) for k, v in self.iter_value_constraints(use_defaults) if k not in attrs
+        ]
         if additional_attrs:
             attrs = {k: v for k, v in attrs.items()}
             attrs.update(additional_attrs)
 
+        if self.xsd_version == '1.0':
+            kwargs['id_list'] = []
+
         filler = kwargs.get('filler')
         result_list = []
         for name, value in attrs.items():
@@ -648,9 +665,13 @@ class XsdAttributeGroup(MutableMapping, XsdComponent, ValidationMixin):
                     try:
                         xsd_attribute = self.maps.lookup_attribute(name)
                     except LookupError:
-                        reason = "%r is not an attribute of the XSI namespace." % name
-                        yield self.validation_error(validation, reason, attrs, **kwargs)
-                        continue
+                        try:
+                            xsd_attribute = self[None]  # None key ==> anyAttribute
+                            value = (name, value)
+                        except KeyError:
+                            reason = "%r is not an attribute of the XSI namespace." % name
+                            yield self.validation_error(validation, reason, attrs, **kwargs)
+                            continue
                 else:
                     try:
                         xsd_attribute = self[None]  # None key ==> anyAttribute
@@ -675,7 +696,7 @@ class XsdAttributeGroup(MutableMapping, XsdComponent, ValidationMixin):
                     result_list.append((name, result))
                     break
 
-        if kwargs.get('fill_missing') is True:
+        if kwargs.get('fill_missing'):
             if filler is None:
                 result_list.extend((k, None) for k in self._attribute_group
                                    if k is not None and k not in attrs)
@@ -694,7 +715,9 @@ class XsdAttributeGroup(MutableMapping, XsdComponent, ValidationMixin):
             yield self.validation_error(validation, reason, attrs, **kwargs)
 
         use_defaults = kwargs.get('use_defaults', True)
-        additional_attrs = [(k, v) for k, v in self.iter_predefined(use_defaults) if k not in attrs]
+        additional_attrs = [
+            (k, v) for k, v in self.iter_value_constraints(use_defaults) if k not in attrs
+        ]
         if additional_attrs:
             attrs = {k: v for k, v in attrs.items()}
             attrs.update(additional_attrs)
@@ -709,9 +732,13 @@ class XsdAttributeGroup(MutableMapping, XsdComponent, ValidationMixin):
                     try:
                         xsd_attribute = self.maps.lookup_attribute(name)
                     except LookupError:
-                        reason = "%r is not an attribute of the XSI namespace." % name
-                        yield self.validation_error(validation, reason, attrs, **kwargs)
-                        continue
+                        try:
+                            xsd_attribute = self[None]  # None key ==> anyAttribute
+                            value = (name, value)
+                        except KeyError:
+                            reason = "%r is not an attribute of the XSI namespace." % name
+                            yield self.validation_error(validation, reason, attrs, **kwargs)
+                            continue
                 else:
                     try:
                         xsd_attribute = self[None]  # None key ==> anyAttribute
