@@ -1,6 +1,5 @@
-# -*- coding: utf-8 -*-
 #
-# Copyright (c), 2016-2019, SISSA (International School for Advanced Studies).
+# Copyright (c), 2016-2020, SISSA (International School for Advanced Studies).
 # All rights reserved.
 # This file is distributed under the terms of the MIT License.
 # See the file 'LICENSE' in the root directory of the present
@@ -8,8 +7,6 @@
 #
 # @author Davide Brunato <brunato@sissa.it>
 #
-from __future__ import unicode_literals
-
 from ..qnames import XSD_NOTATION, get_qname
 from .xsdbase import XsdComponent
 
@@ -43,7 +40,7 @@ class XsdNotation(XsdComponent):
             self.parse_error("a notation must have a 'name' attribute", self.elem)
 
         if 'public' not in self.elem.attrib and 'system' not in self.elem.attrib:
-            self.parse_error("a notation must has a 'public' or a 'system' attribute", self.elem)
+            self.parse_error("a notation must have a 'public' or a 'system' attribute", self.elem)
 
     @property
     def public(self):
