@@ -20,8 +20,8 @@ from ..qnames import XSD_ANY_TYPE, XSD_SIMPLE_TYPE, XSD_ANY_ATOMIC_TYPE, \
     XSD_LENGTH, XSD_MIN_LENGTH, XSD_MAX_LENGTH, XSD_WHITE_SPACE, XSD_ENUMERATION,\
     XSD_LIST, XSD_ANY_SIMPLE_TYPE, XSD_UNION, XSD_RESTRICTION, XSD_ANNOTATION, \
     XSD_ASSERTION, XSD_ID, XSD_IDREF, XSD_FRACTION_DIGITS, XSD_TOTAL_DIGITS, \
-    XSD_EXPLICIT_TIMEZONE, XSD_ERROR, XSD_ASSERT, XSD_QNAME, get_qname, local_name, \
-    is_not_xsd_annotation
+    XSD_EXPLICIT_TIMEZONE, XSD_ERROR, XSD_ASSERT, XSD_QNAME, XSD_UNTYPED_ATOMIC, \
+    get_qname, get_prefixed_qname, local_name, is_not_xsd_annotation
 from ..namespaces import XSD_NAMESPACE
 from ..helpers import get_xsd_derivation_attribute
 
@@ -327,6 +327,24 @@ class XsdSimpleType(XsdType, ValidationMixin):
     def content_type_label(self):
         return 'empty' if self.max_length == 0 else 'simple'
 
+    @property
+    def sequence_type(self):
+        if self.is_empty():
+            return 'empty-sequence()'
+
+        root_type = self.root_type
+        if root_type.name is None:
+            sequence_type = get_prefixed_qname(XSD_UNTYPED_ATOMIC, self.namespaces)
+        else:
+            sequence_type = root_type.prefixed_name
+
+        if not self.is_list():
+            return sequence_type
+        elif self.is_emptiable():
+            return '{}*'.format(sequence_type)
+        else:
+            return '{}+'.format(sequence_type)
+
     def is_empty(self):
         return self.max_length == 0
 
@@ -368,7 +386,7 @@ class XsdSimpleType(XsdType, ValidationMixin):
             return self.base_type.is_derived(other, derivation)
 
     def is_dynamic_consistent(self, other):
-        return other.name in (XSD_ANY_TYPE, XSD_ANY_SIMPLE_TYPE) or self.is_derived(other) or \
+        return other.name in {XSD_ANY_TYPE, XSD_ANY_SIMPLE_TYPE} or self.is_derived(other) or \
             hasattr(other, 'member_types') and any(self.is_derived(mt) for mt in other.member_types)
 
     def normalize(self, text):
@@ -749,7 +767,7 @@ class XsdList(XsdSimpleType):
                 base_type = xsd_simple_type_factory(child, self.schema, self)
             except XMLSchemaParseError as err:
                 self.parse_error(err, elem)
-                base_type = self.maps.types[XSD_ANY_ATOMIC_TYPE]
+                base_type = self.any_atomic_type
 
             if 'itemType' in elem.attrib:
                 self.parse_error("ambiguous list type declaration", self)
@@ -763,33 +781,33 @@ class XsdList(XsdSimpleType):
                     self.parse_error("missing list type declaration")
                 else:
                     self.parse_error(err)
-                base_type = self.maps.types[XSD_ANY_ATOMIC_TYPE]
+                base_type = self.any_atomic_type
             else:
                 try:
                     base_type = self.maps.lookup_type(item_qname)
                 except KeyError:
                     self.parse_error("unknown itemType %r" % elem.attrib['itemType'], elem)
-                    base_type = self.maps.types[XSD_ANY_ATOMIC_TYPE]
+                    base_type = self.any_atomic_type
                 else:
                     if isinstance(base_type, tuple):
                         self.parse_error(
                             "circular definition found for type {!r}".format(item_qname)
                         )
-                        base_type = self.maps.types[XSD_ANY_ATOMIC_TYPE]
+                        base_type = self.any_atomic_type
 
         if base_type.final == '#all' or 'list' in base_type.final:
             self.parse_error(
                 "'final' value of the itemType %r forbids derivation by list" % base_type
             )
 
-        if base_type is self.any_atomic_type:
+        if base_type.name == XSD_ANY_ATOMIC_TYPE:
             self.parse_error("Cannot use xs:anyAtomicType as base type of a user-defined type")
 
         try:
             self.base_type = base_type
         except XMLSchemaValueError as err:
             self.parse_error(str(err), elem)
-            self.base_type = self.maps.types[XSD_ANY_ATOMIC_TYPE]
+            self.base_type = self.any_atomic_type
         else:
             if not base_type.allow_empty and self.min_length != 0:
                 self.allow_empty = False
@@ -923,10 +941,10 @@ class XsdUnion(XsdSimpleType):
                     mt = self.maps.lookup_type(type_qname)
                 except KeyError:
                     self.parse_error("unknown member type %r" % type_qname)
-                    mt = self.maps.types[XSD_ANY_ATOMIC_TYPE]
+                    mt = self.any_atomic_type
                 except XMLSchemaParseError as err:
                     self.parse_error(err)
-                    mt = self.maps.types[XSD_ANY_ATOMIC_TYPE]
+                    mt = self.any_atomic_type
 
                 if isinstance(mt, tuple):
                     self.parse_error(
@@ -944,8 +962,8 @@ class XsdUnion(XsdSimpleType):
 
         if not member_types:
             self.parse_error("missing xs:union type declarations", elem)
-            self.member_types = [self.maps.types[XSD_ANY_ATOMIC_TYPE]]
-        elif any(mt is self.any_atomic_type for mt in member_types):
+            self.member_types = [self.any_atomic_type]
+        elif any(mt.name == XSD_ANY_ATOMIC_TYPE for mt in member_types):
             self.parse_error("Cannot use xs:anyAtomicType as base type of a user-defined type")
         else:
             self.member_types = member_types
@@ -962,11 +980,14 @@ class XsdUnion(XsdSimpleType):
     def is_list(self):
         return all(mt.is_list() for mt in self.member_types)
 
+    def is_key(self):
+        return any(mt.is_key() for mt in self.member_types)
+
     def is_union(self):
         return True
 
     def is_dynamic_consistent(self, other):
-        return other.name in (XSD_ANY_TYPE, XSD_ANY_SIMPLE_TYPE) or \
+        return other.name in {XSD_ANY_TYPE, XSD_ANY_SIMPLE_TYPE} or \
             other.is_derived(self) or isinstance(other, self.__class__) and \
             any(mt1.is_derived(mt2) for mt1 in other.member_types for mt2 in self.member_types)
 
@@ -1067,8 +1088,8 @@ class XsdAtomicRestriction(XsdAtomic):
           enumeration | whiteSpace | pattern)*))
         </restriction>
     """
-    FACETS_BUILDERS = XSD_10_FACETS_BUILDERS
     derivation = 'restriction'
+    _FACETS_BUILDERS = XSD_10_FACETS_BUILDERS
     _CONTENT_TAIL_TAGS = {XSD_ATTRIBUTE, XSD_ATTRIBUTE_GROUP, XSD_ANY_ATTRIBUTE}
 
     def __setattr__(self, name, value):
@@ -1101,12 +1122,12 @@ class XsdAtomicRestriction(XsdAtomic):
                 base_qname = self.schema.resolve_qname(elem.attrib['base'])
             except (KeyError, ValueError, RuntimeError) as err:
                 self.parse_error(err, elem)
-                base_type = self.maps.type[XSD_ANY_ATOMIC_TYPE]
+                base_type = self.any_atomic_type
             else:
                 if base_qname == self.name:
                     if self.redefine is None:
                         self.parse_error("wrong definition with self-reference", elem)
-                        base_type = self.maps.type[XSD_ANY_ATOMIC_TYPE]
+                        base_type = self.any_atomic_type
                     else:
                         base_type = self.base_type
                 else:
@@ -1117,15 +1138,15 @@ class XsdAtomicRestriction(XsdAtomic):
                         base_type = self.maps.lookup_type(base_qname)
                     except KeyError:
                         self.parse_error("unknown type %r." % elem.attrib['base'])
-                        base_type = self.maps.types[XSD_ANY_ATOMIC_TYPE]
+                        base_type = self.any_atomic_type
                     except XMLSchemaParseError as err:
                         self.parse_error(err)
-                        base_type = self.maps.types[XSD_ANY_ATOMIC_TYPE]
+                        base_type = self.any_atomic_type
                     else:
                         if isinstance(base_type, tuple):
                             msg = "circularity definition between %r and %r"
                             self.parse_error(msg % (self, base_qname), elem)
-                            base_type = self.maps.types[XSD_ANY_ATOMIC_TYPE]
+                            base_type = self.any_atomic_type
 
             if base_type.is_simple() and base_type.name == XSD_ANY_SIMPLE_TYPE:
                 self.parse_error("wrong base type {!r}, an atomic type required")
@@ -1161,7 +1182,7 @@ class XsdAtomicRestriction(XsdAtomic):
                         base_type = xsd_simple_type_factory(child, self.schema, self)
                     except XMLSchemaParseError as err:
                         self.parse_error(err)
-                        base_type = self.maps.types[XSD_ANY_SIMPLE_TYPE]
+                        base_type = self.any_simple_type
                 elif base_type.is_complex():
                     if base_type.admit_simple_restriction():
                         base_type = self.schema.BUILDERS.complex_type_class(
@@ -1182,7 +1203,7 @@ class XsdAtomicRestriction(XsdAtomic):
                 has_simple_type_child = True
             else:
                 try:
-                    facet_class = self.FACETS_BUILDERS[child.tag]
+                    facet_class = self._FACETS_BUILDERS[child.tag]
                 except KeyError:
                     self.parse_error("unexpected tag %r in restriction:" % child.tag)
                     continue
@@ -1314,5 +1335,5 @@ class Xsd11AtomicRestriction(XsdAtomicRestriction):
           {any with namespace: ##other})*))
         </restriction>
     """
-    FACETS_BUILDERS = XSD_11_FACETS_BUILDERS
+    _FACETS_BUILDERS = XSD_11_FACETS_BUILDERS
     _CONTENT_TAIL_TAGS = {XSD_ATTRIBUTE, XSD_ATTRIBUTE_GROUP, XSD_ANY_ATTRIBUTE, XSD_ASSERT}
