@@ -15,7 +15,8 @@ import re
 from ..exceptions import XMLSchemaValueError, XMLSchemaTypeError
 from ..qnames import XSD_ANNOTATION, XSD_APPINFO, XSD_DOCUMENTATION, XML_LANG, \
     XSD_ANY_TYPE, XSD_ANY_SIMPLE_TYPE, XSD_ANY_ATOMIC_TYPE, XSD_ID, XSD_QNAME, \
-    XSD_OVERRIDE, get_qname, local_name, qname_to_prefixed, is_not_xsd_annotation
+    XSD_OVERRIDE, XSD_NOTATION_TYPE, get_qname, local_name, get_prefixed_qname, \
+    is_not_xsd_annotation
 from ..etree import etree_tostring
 from ..helpers import is_etree_element
 from .exceptions import XMLSchemaParseError, XMLSchemaValidationError
@@ -232,6 +233,7 @@ class XsdComponent(XsdValidator):
         if parent is not None:
             self.parent = parent
         self.schema = schema
+        self.maps = schema.maps
         self.elem = elem
 
     def __setattr__(self, name, value):
@@ -297,25 +299,20 @@ class XsdComponent(XsdValidator):
         """Property that references to schema's namespace mapping."""
         return self.schema.namespaces
 
-    @property
-    def maps(self):
-        """Property that references to schema's global maps."""
-        return self.schema.maps
-
     @property
     def any_type(self):
         """Property that references to the xs:anyType instance of the global maps."""
-        return self.schema.maps.types[XSD_ANY_TYPE]
+        return self.maps.types[XSD_ANY_TYPE]
 
     @property
     def any_simple_type(self):
         """Property that references to the xs:anySimpleType instance of the global maps."""
-        return self.schema.maps.types[XSD_ANY_SIMPLE_TYPE]
+        return self.maps.types[XSD_ANY_SIMPLE_TYPE]
 
     @property
     def any_atomic_type(self):
         """Property that references to the xs:anyAtomicType instance of the global maps."""
-        return self.schema.maps.types[XSD_ANY_ATOMIC_TYPE]
+        return self.maps.types[XSD_ANY_ATOMIC_TYPE]
 
     def __repr__(self):
         if self.name is None:
@@ -434,7 +431,7 @@ class XsdComponent(XsdValidator):
     @property
     def prefixed_name(self):
         """The name of the component in prefixed format, or `None` if the name is `None`."""
-        return qname_to_prefixed(self.name, self.namespaces)
+        return get_prefixed_qname(self.name, self.namespaces)
 
     @property
     def id(self):
@@ -518,12 +515,9 @@ class XsdComponent(XsdValidator):
         ancestor = self
         while True:
             ancestor = ancestor.parent
-            if ancestor is None:
-                break
-            elif xsd_classes is None or isinstance(ancestor, xsd_classes):
-                yield ancestor
-            else:
+            if ancestor is None or xsd_classes and not isinstance(ancestor, xsd_classes):
                 break
+            yield ancestor
 
     def tostring(self, indent='', max_lines=None, spaces_for_tab=4):
         """Serializes the XML elements that declare or define the component to a string."""
@@ -602,11 +596,22 @@ class XsdType(XsdComponent):
         """The content type classification."""
         raise NotImplementedError()
 
+    @property
+    def sequence_type(self):
+        """The XPath sequence type associated with the content."""
+        raise NotImplementedError()
+
     @property
     def root_type(self):
-        """The root type of the type definition hierarchy. Is itself for a root type."""
-        if self.base_type is None:
-            return self  # Note that a XsdUnion type is always considered a root type
+        """
+        The root type of the type definition hierarchy. For an atomic type
+        is the primitive type. For a list is the primitive type of the item.
+        For a union is the base union type. For a complex type is xs:anyType.
+        """
+        if self.is_complex() and self.attributes:
+            return self.maps.types[XSD_ANY_TYPE]
+        elif self.base_type is None:
+            return self if self.is_simple() else self.maps.types[XSD_ANY_TYPE]
 
         try:
             if self.base_type.is_simple():
@@ -615,7 +620,7 @@ class XsdType(XsdComponent):
                 return self.base_type.content.primitive_type
         except AttributeError:
             # The type has complex or XsdList content
-            return self.base_type
+            return self.base_type.root_type
 
     @property
     def simple_type(self):
@@ -727,7 +732,8 @@ class XsdType(XsdComponent):
 
     def is_dynamic_consistent(self, other):
         return other.name == XSD_ANY_TYPE or self.is_derived(other) or \
-            hasattr(other, 'member_types') and any(self.is_derived(mt) for mt in other.member_types)
+            hasattr(other, 'member_types') and \
+            any(self.is_derived(mt) for mt in other.member_types)  # pragma: no cover
 
     def is_key(self):
         return self.name == XSD_ID or self.is_derived(self.maps.types[XSD_ID])
@@ -735,6 +741,9 @@ class XsdType(XsdComponent):
     def is_qname(self):
         return self.name == XSD_QNAME or self.is_derived(self.maps.types[XSD_QNAME])
 
+    def is_notation(self):
+        return self.name == XSD_NOTATION_TYPE or self.is_derived(self.maps.types[XSD_NOTATION_TYPE])
+
     def text_decode(self, text):
         raise NotImplementedError()
 
@@ -813,12 +822,12 @@ class ValidationMixin(object):
         check_validation_mode(validation)
 
         result, errors = None, []
-        for result in self.iter_decode(source, validation, **kwargs):
+        for result in self.iter_decode(source, validation, **kwargs):  # pragma: no cover
             if not isinstance(result, XMLSchemaValidationError):
                 break
             elif validation == 'strict':
                 raise result
-            elif validation == 'lax':
+            else:
                 errors.append(result)
 
         return (result, errors) if validation == 'lax' else result
@@ -840,12 +849,12 @@ class ValidationMixin(object):
         check_validation_mode(validation)
 
         result, errors = None, []
-        for result in self.iter_encode(obj, validation=validation, **kwargs):
+        for result in self.iter_encode(obj, validation=validation, **kwargs):  # pragma: no cover
             if not isinstance(result, XMLSchemaValidationError):
                 break
             elif validation == 'strict':
                 raise result
-            elif validation == 'lax':
+            else:
                 errors.append(result)
 
         return (result, errors) if validation == 'lax' else result
@@ -975,10 +984,10 @@ class ParticleMixin(object):
         return self.max_occurs is not None and self.max_occurs <= occurs
 
     def has_occurs_restriction(self, other):
-        if self.min_occurs == self.max_occurs == 0:
-            return True
-        elif self.min_occurs < other.min_occurs:
+        if self.min_occurs < other.min_occurs:
             return False
+        elif self.max_occurs == 0:
+            return True
         elif other.max_occurs is None:
             return True
         elif self.max_occurs is None:
@@ -987,7 +996,7 @@ class ParticleMixin(object):
             return self.max_occurs <= other.max_occurs
 
     def parse_error(self, message):
-        raise XMLSchemaParseError(self, message)
+        raise XMLSchemaParseError(self, message)   # pragma: no cover
 
     def _parse_particle(self, elem):
         if 'minOccurs' in elem.attrib:
